@@ -8,6 +8,7 @@ from collections import Counter
 import numpy as np
 import sympy
 from scipy import optimize
+from sklearn.utils import resample
 from sympy import Function
 from sympy.core.function import UndefinedFunction
 from sympy.core.numbers import NegativeOne
@@ -470,7 +471,7 @@ def cla(pre_y, cl=True):
     return pre_y
 
 
-def try_add_coef(expr01, x, y, terminals,
+def try_add_coef(expr01, x, y, terminals, grid_x=None,
                  filter_warning=True, inter_add=True, inner_add=False, vector_add=False, out_add=False, flat_add=False,
                  np_maps=None, classification=False):
     """
@@ -490,6 +491,9 @@ def try_add_coef(expr01, x, y, terminals,
         list of xi
     y: np.ndarray
         y value
+
+    grid_x:
+        new x to predict
     terminals: list of sympy.Symbol
         features and constants
     filter_warning: bool
@@ -559,9 +563,14 @@ def try_add_coef(expr01, x, y, terminals,
         cof = result.x
 
         cof = cc.group(cof)
-        pre_y = func0(*x + cof)
+
+        if grid_x is None:
+            grid_x=x
+
+        pre_y = func0(*grid_x + cof)
         if classification:
             pre_y = cla(pre_y, cl=True)
+
         cof = cc.dec(cof)
 
         for ai, choi in zip(cc.name, cof):
@@ -584,3 +593,113 @@ def try_add_coef(expr01, x, y, terminals,
         pre_y = None
 
     return pre_y, expr01
+
+
+def try_add_coef_times(expr01, x, y, terminals, grid_x=None,
+                       filter_warning=True, inter_add=True, inner_add=False, vector_add=False, out_add=False,
+                       flat_add=False,
+                       np_maps=None, classification=False, random_state=0, return_expr=False,resample_number=500):
+    if filter_warning:
+        warnings.filterwarnings("ignore")
+
+    if grid_x is None:
+        grid_x = x
+
+    expr01, a_list, a_dict = add_coefficient(expr01, inter_add=inter_add, inner_add=inner_add, vector_add=vector_add,
+                                             out_add=out_add, flat_add=flat_add)
+
+    cc = CheckCoef(a_list, a_dict)
+
+    ter = []
+    ter.extend(terminals)
+    ter.extend(cc.name)
+
+    func0 = sympy.utilities.lambdify(ter, expr01, modules=[np_maps, "numpy", "math"])
+
+    def func(x_, p):
+        """"""
+        num_list = []
+        num_list.extend(x_)
+
+        p = cc.group(p)
+        num_list.extend(p)
+
+        return func0(*num_list)
+
+    def res(p, x_, y_):
+        """"""
+        ress = y_ - func(x_, p)
+        return ress
+
+    def res2(p, x_, y_):
+        """"""
+        pre_y = func(x_, p)
+        pre_y = cla(pre_y, cl=False)
+        ress = y_ - pre_y
+
+        return ress
+
+    pre_y_all = []
+    expr_all = []
+    for times in range(resample_number):
+        *x, y = resample(*x,  y, replace=True, random_state=times)
+
+        if not classification:
+            result = optimize.least_squares(res, x0=[1.0] * cc.num, args=(x, y), xtol=1e-4, ftol=1e-4, gtol=1e-4,
+                                            # max_nfev=200,
+                                            jac='3-point', loss='linear')
+        else:
+            result = optimize.least_squares(res2, x0=[1.0] * cc.num, args=(x, y), xtol=1e-4, ftol=1e-4, gtol=1e-4,
+                                            # max_nfev=200,
+                                            jac='3-point', loss='linear')
+        cof = result.x
+        cof = cc.group(cof)
+
+        try:
+
+            pre_y = func0(*grid_x + cof)
+
+            if classification:
+                pre_y = cla(pre_y, cl=True)
+
+                expr00 = expr01
+            if return_expr:
+
+                expr00 = copy.deepcopy(expr01)
+
+                cof = cc.dec(cof)
+                for ai, choi in zip(cc.name, cof):
+                    if ai in cc.cof_dict_keys:
+
+                        fun = Coef(ai.name, choi)
+                        # replace the Vi to Vi()
+                        olds0 = find_args(expr00, ai)
+                        if olds0 is None:
+                            raise KeyError("0*wi is 0,and make the placeholder fade out")
+                        olds = [old for old in olds0 if old is not ai]
+                        olds = sympy.Mul(*olds)
+                        expr00 = expr00.xreplace({sympy.Mul(ai, olds): fun(olds)})
+                    else:
+                        expr00 = expr00.xreplace({ai: choi})
+            else:
+                pass
+
+        except (ValueError, KeyError, NameError, TypeError, ZeroDivisionError):
+
+            pre_y = None
+
+        if isinstance(pre_y, np.ndarray):
+
+            if np.all(np.isfinite(pre_y)):
+                pre_y_all.append(pre_y)
+
+                if return_expr:
+                    expr_all.append(expr00)
+
+    pre_y_all = np.array(pre_y_all)
+
+    if return_expr:
+        return pre_y_all, expr_all
+    else:
+
+        return pre_y_all
